from botiverse.bots.BasicBot.BasicBot import BasicBot
from botiverse.bots.WhizBot.WhizBot import WhizBot
from botiverse.bots.ConverseBot.ConverseBot import ConverseBot
from botiverse.bots.basic_TODS.basic_TODS import BasicTODS
from botiverse.bots.deep_TODS.deep_TODS import DeepTODS
from botiverse.bots.VoiceBot.SpeechClassifier import SpeechClassifier
from botiverse.bots.VoiceBot.VoiceBot import VoiceBot
